@@ -6,8 +6,7 @@ from sklearn import linear_model
 import scipy.stats as stats
 import seaborn as sns
 #from utils import compute_pathway
-# from .utils import *
-from utils import * #TODO .utils
+from .utils import *
 import holoviews as hv
 from holoviews import opts, dim
 from bokeh.io import output_file, show
